clear
close all
clc

set(0,'defaultlinelinewidth',2)
set(0,'defaultaxesfontsize',14)
set(0,'defaulttextfontsize',14)

%Important you can only do this with the TRAINING SET here

NF_is_on = 1;

if NF_is_on %Training
    names_of_signals = {'NF_TRAINING_T_FC_min.dat','NF_TRAINING_T_FC_max.dat',...
        'NF_TRAINING_T_IC_min.dat','NF_TRAINING_T_IC_max.dat'};
else
    names_of_signals = {'TRAINING_T_FC_min.dat','TRAINING_T_FC_max.dat',...
        'TRAINING_T_IC_min.dat','TRAINING_T_IC_max.dat'};
end

list_of_titles = {'T_{FC} inlet','T_{FC} outlet','T_{IC} inlet','T_{IC} outlet'};

for jj = 1:numel(names_of_signals)
    
    Model_Out = load(names_of_signals{jj});
    
    Ntransients = size(Model_Out,1);
    
    var_vector = [0.99 0.999 0.9999 0.99999];
    
    for kk = 1:length(var_vector)
        
        perc_var_req1 = var_vector(kk);
        
        %perc_var_req1 % percentuale di variabilit del dataset che noi vogliamo "spiegare" dopo il troncamento dello sviluppo ad un numero finito (e possibilimente ridotto) di basi ortogonali
        
        medie = mean(Model_Out,1); % Model_out contiene i transitori (Ntransients * TimeSteps) --> calcolo media dei transitori
        data_reduced = Model_Out - ones(Ntransients,1)*medie; %--> sottrarre la media ai transitori
        [U1, S1, V1] = svd(data_reduced); % decomposizione ai valori singolari (SVD)
        coefficient_data = data_reduced*V1; % proiezione dei transitori ridotti sulle basi ortonormali --> si ottengono coefficienti indipendenti dal tempo e dipendenti solo dagli input
        % le colonne di V1 (TimeSteps * TimeSteps) contengono le basi ortornormali
        singval1 = diag(S1); % --> estrazione della diagonale di S1: contiene gli autovalori della decomposizione
        perc_var1 = cumsum(singval1.^2)/sum(singval1.^2); % vettore che contiene la "percentuale di varianza" spiegata dagli autovalori
        nbase1 = min(find(perc_var1 >= perc_var_req1)); % si tronca la decomposizione ad un numero di basi (nbase1) tali che spieghino una percentuale voluta di varianza (es. perc_var_req1 = 99%)
        % % NOTA: coefficient_data(:, 1:nbase1) sono i punti nbase1-dimensionali
        % che tu tenterai di classificare!
        
   
%         reconstructed_transients = coefficient_data(:, 1:nbase1)*V1(:, 1:nbase1)' + ones(Ntransients,1)*medie;
%         integral_rel_error(kk) = norm(Model_Out-reconstructed_transients,"fro")/;
        
        num_coeff(jj,kk) = size(coefficient_data(:, 1:nbase1),2);
    end
    
    fig = figure(1)
    subplot(2,2,jj)
    plot(num_coeff(jj,:),100*var_vector,'-o')
    grid on
    title(list_of_titles{jj})
   
end

 % Give common xlabel, ylabel and title to your figure
 han=axes(fig,'visible','off');
 han.XLabel.Visible='on';
 han.YLabel.Visible='on';
 ax.YTickLabel={'99','99.9','99.99','99.999'};
%  ax.YAxis.TickLabelFormat = '%.4f';
 xlabel('Number of basis (-)')
 ylabel('PVE (%)')


%% Plotting some visual reconstruction

list_of_titles = {'T_{FC} inlet','T_{FC} outlet','T_{IC} inlet','T_{IC} outlet'};

for jj = 1:numel(names_of_signals)
    
    Model_Out = load(names_of_signals{jj}); %What to decompose
    perc_var_req1 = 0.99999; %What precision to use
    
    medie = mean(Model_Out,1); % Model_out contiene i transitori (Ntransients * TimeSteps) --> calcolo media dei transitori
    data_reduced = Model_Out - ones(Ntransients,1)*medie; %--> sottrarre la media ai transitori
    [U1, S1, V1] = svd(data_reduced); % decomposizione ai valori singolari (SVD)
    coefficient_data = data_reduced*V1; % proiezione dei transitori ridotti sulle basi ortonormali --> si ottengono coefficienti indipendenti dal tempo e dipendenti solo dagli input
    % le colonne di V1 (TimeSteps * TimeSteps) contengono le basi ortornormali
    singval1 = diag(S1); % --> estrazione della diagonale di S1: contiene gli autovalori della decomposizione
    perc_var1 = cumsum(singval1.^2)/sum(singval1.^2); % vettore che contiene la "percentuale di varianza" spiegata dagli autovalori
    nbase1 = min(find(perc_var1 >= perc_var_req1)); % si tronca la decomposizione ad un numero di basi (nbase1) tali che spieghino una percentuale voluta di varianza (es. perc_var_req1 = 99%)
    % % NOTA: coefficient_data(:, 1:nbase1) sono i punti nbase1-dimensionali
    % che tu tenterai di classificare!
    
    reconstructed_transients = coefficient_data(:, 1:nbase1)*V1(:, 1:nbase1)' + ones(Ntransients,1)*medie;
    
    tt = [1:1:806];
    time = ones(806,1)*tt;
    
    fig = figure(2)
    subplot(2,2,jj)
   
    for ii = 1:300:size(reconstructed_transients,1)
        p1 = plot(time(1,:), Model_Out(ii,:),'-b')
        hold on
    end
    hold on
    for ii = 1:300:size(reconstructed_transients,1)
        p2 = plot(time(1,:), reconstructed_transients(ii,:),'--r')
        hold on
    end
    hold off
    grid on
    title(list_of_titles{jj})
end

 % Give common xlabel, ylabel and title to your figure
 han=axes(fig,'visible','off');
 han.XLabel.Visible='on';
 han.YLabel.Visible='on';
 
 xlabel('Time (s)')
 ylabel('Temperature (K)')


legend([p1(1) p2(1)],'Original signals','SVD reconstruction','Orientation','horizontal','Location','southoutside')