clear
close all
clc

set(0,'defaultlinelinewidth',2)
set(0,'defaultaxesfontsize',14)
set(0,'defaulttextfontsize',14)

%Version 2 includes subplots
%Autosave

%%%%%%%% SCRIPT POURPOSE %%%%%%%%%%

%Plot the # of observations in each class (TRAINING SET)
%Plot the probability distribution functions for each class (TRAINING SET)
%Plot the probability distribution functions for 1 observation (TEST SET)

%%%%%%%% SCRIPT POURPOSE %%%%%%%%%%

%Legend:
%FI: Failure Intensity 
%pump 1: Fuel Circuit pump
%pump 2: Intermediate Cicuit pump
%pump 3: Gac Circuit pump

%%%%%%%%%%%%%%%%% CONTROL PANEL %%%%%%%%%%%%%%%%%%%%%%%%

%Suffissi da mettere al nome del file da gestire
category = '_C3';            %C1 C2 C3
data_set = '';            %_NF or empty
ToT = '_TRAINING';        %_TRAINING or _TEST
%%%%%%%%%%%%%%%%% CONTROL PANEL %%%%%%%%%%%%%%%%%%%%%%%%

category_name = {erase(category,'_')};
ext_txt = '.txt';
ext_jpg = '.jpg';

%Settings to auto save figures
path = 'C:\Users\NICAR\OneDrive\Desktop';
base_name_fig1 = '\fig_num_obs_train';
base_name_fig2 = '\fig_probs_train';
base_name_fig3 = '\fig_probs_test_point';

%Assembling the names
name_sim = ['input' ToT data_set ext_txt]; %Training set: points
name_labels = ['labels' category ToT data_set ext_txt]; %correct labels of the TRAINING SET
name_sim_test = ['input_TEST' data_set ext_txt]; %Test set: points

%%%%%%%%% Loading the input points %%%%%%%%%%% 

%Loading the points of the TRAINING SET
fileName = name_sim;
FID = fopen(fileName);
data = textscan(FID,'%s','Delimiter',{','});
sim_codes = string(data{:});
sim_codes = reshape( sim_codes , 3 , [])';
fclose(FID);
sim_codes = str2double(sim_codes); %Convert the string matrix in a matrix of doubles

%Loading the associated labels
fileName = name_labels; 
FID = fopen(fileName);
data = textscan(FID,'%s','Delimiter',{'\n'});
labels = string(data{:});
fclose(FID);
labels = categorical(labels);

%Loading the points of the TEST SET
fileName = name_sim_test;
FID = fopen(fileName);
data = textscan(FID,'%s','Delimiter',{','});
sim_codes_TEST = string(data{:});
sim_codes_TEST  = reshape( sim_codes_TEST  , 3 , [])';
fclose(FID);
sim_codes_TEST  = str2double(sim_codes_TEST ); %Convert the string matrix in a matrix of doubles

%Loading the scores associated to the points of the TEST set
tmp = load('score_TEST_C3.mat');
score = tmp.score; %Contains all the scores for the TEST set
score = [score(:,end),score(:,2),score(:,1)]; %IMPORTANT THE ORDER SHOULD BE COOHERENT WITH LEGEND LIST ORDER

%%%%%%%%% End of loading %%%%%%%%%%% 


%% Computing part

%Make the legend list from the labels in input
legend_list = unique(labels);

% Shift to correct range
FI_pump1 = -100*sim_codes(:,1); 
FI_pump2 = -100*sim_codes(:,2);
FI_pump3 = -100*sim_codes(:,3);


%Some usefull structures
nbins = 10; %number of bins for the histograms
edges = (-100:10:0); %Define the limits for the edges of the bins
colors = [0 0.4470 0.7410;0.8500 0.3250 0.0980;0.9290 0.6940 0.1250];
failure_names = ['Failure Intensity FC'; 'Failure Intensity IC'; 'Failure Intensity GC'];

prob_diff_classes_a = zeros(length(legend_list),length(edges)-1);
prob_diff_classes_b = zeros(length(legend_list),length(edges)-1);
prob_diff_classes_c = zeros(length(legend_list),length(edges)-1);

for jj = 1:length(legend_list)
     
      wts1 = [ path base_name_fig1 category data_set strcat('_',char(legend_list(jj))) ext_jpg];
      wts2 = [ path base_name_fig2 category data_set strcat('_',char(legend_list(jj))) ext_jpg];

      
      %jj goes through all possible classes
    
      % Get the index of the current class:
      ind = labels==legend_list(jj); 
      a = FI_pump1(ind);
      b = FI_pump2(ind);
      c = FI_pump3(ind);
      
      pumps = [a,b,c]; 
      
      fig_num_obs = figure;
      for kk = 1:3 
          %Plot the # of observations in the jj class for each pump (TRAINING SET)
          subplot(1,3,kk)
          histogram(pumps(:,kk),nbins,'FaceColor',num2str(colors(kk,:)))
          xlabel(failure_names(kk,:))
          xlim([-100;0])
          % Give common: ylabel and title to your figure
          han=axes(fig_num_obs,'visible','off');
          han.Title.Visible='on';
          han.YLabel.Visible='on';
          ylabel(han,'# of observations')
          title(han,strcat({'Category: '}, category_name, {'| Class Name: '}, cellstr(legend_list(jj))))
      end
      saveas(fig_num_obs,wts1)
      
      N_prob_matrix = [];
      for kk = 1:3
          N_prob = histcounts(pumps(:,kk),edges,'Normalization', 'probability'); %Normalization step
          N_prob_matrix = [N_prob_matrix; N_prob];
      end
      
      fig_probs = figure;
      for kk = 1:3
          subplot(1,3,kk)
          %Plot the probability distribution functions in the jj class for each pump (TRAINING SET)
          histogram('BinEdges',edges,'BinCounts',N_prob_matrix(kk,:),'FaceColor',num2str(colors(kk,:)))
          xlabel(failure_names(kk,:))
          xlim([-100;0])
          % Give common: ylabel and title to your figure
          han=axes(fig_probs,'visible','off');
          han.Title.Visible='on';
          han.YLabel.Visible='on';
          
          ylabel(han,'Probability (%)')
          title(han,strcat({'Category: '}, category_name, {'| Class Name: '}, cellstr(legend_list(jj))))
          
      end
      saveas(fig_probs,wts2)
      
      prob_diff_classes_a(jj,:)=N_prob_matrix(1,:);
      prob_diff_classes_b(jj,:)=N_prob_matrix(2,:);
      prob_diff_classes_c(jj,:)=N_prob_matrix(3,:);
        
end

%% 3 colonna 0.5714 classificato come successo oppure 0.4286 classificato come doppio LT

%Interesting positions:
%find(score(:,2)>0.85 & score(:,3)>0.14) || num_sim = 1104 || score [0    0.8571    0.1429]
%find(score(:,2)>0.14 & score(:,3)>0.85) || num_sim = 999 || score[0    0.1429    0.8571]
%find(score(:,1)>0.14 & score(:,2)>0.18 & score(:,3)>0.57) || num_sim = 1334 || score[0.1429    0.2857    0.5714]
%find(score(:,1)>0.71 & score(:,2)>0.28)|| num_sim = 522 || score  

num_sim = 522;
test_obs = -100*sim_codes_TEST(num_sim,:); %Choose an observation point in the test set
score_obs = score(num_sim,:)'; %Retrive the associated probabilities of belonging in the different classes

new_bin_count_a = score_obs.*prob_diff_classes_a;
new_bin_count_b = score_obs.*prob_diff_classes_b;
new_bin_count_c = score_obs.*prob_diff_classes_c;

% Order
%      LT_{FC}LT_{IC} 
%      LT_{IC} 
%      Success 

new_bin_count_matrix = [sum(new_bin_count_a);sum(new_bin_count_b);sum(new_bin_count_c)];

fig_probs_test_point = figure;
wts3 = [ path base_name_fig3 category data_set char '_' (num2str(num_sim)) ext_jpg];
for kk = 1:3
    
    %Plot the probability distribution functions for 1 observation point of the (TEST SET)
    subplot(1,3,kk)
    histogram('BinEdges',edges,'BinCounts',new_bin_count_matrix(kk,:),'FaceColor',num2str(colors(kk,:)))
    xlabel(failure_names(kk,:))
    xlim([-100;0])
    % Give common: ylabel and title to your figure
    han=axes(fig_probs_test_point,'visible','off');
    han.Title.Visible='on';
    han.YLabel.Visible='on';
    
    ylabel(han,'Probability (%)')
    title(han,strcat({'Category: '}, category_name, '| Observation point: (',num2str(test_obs(1)),',',...
    num2str(test_obs(2)),',',num2str(test_obs(3)),')%' ))
    
    saveas(fig_probs_test_point,wts3)
end

disp(score_obs)
