clear
close all
clc 

set(0,'defaultlinelinewidth',2)
set(0,'defaultaxesfontsize',14)
set(0,'defaulttextfontsize',14)

%Normalization is performed

%Performs SVD decomposition in order to obtain a
%matrix which is (Ntransients x NcoeffOfSVD) which condenses all the info
%of the 4 output signals in a number of columns equal to NcoeffOfSVD
%Calling the SVD function in order to reduce the # of columns of the
%data-set for the training of the classifier

%Please choose if you want to produce the SVD decomposition with NF data or
%note by changing the value to NF to 1 or 0
NF_is_on = 0;

if NF_is_on
    
    %%%%%%%%%%% Loading the 4 outputs of the simulations NF on %%%%%%%%%%%%
    T_FC_1 = load('NF_TRAINING_T_FC_min.dat');
    T_FC_2 = load('NF_TRAINING_T_FC_max.dat');
    T_IC_3 = load('NF_TRAINING_T_IC_min.dat');
    T_IC_4 = load('NF_TRAINING_T_IC_max.dat');
    
    %%%%%%%%%%%% Loading the TEST 4 outputs of the simulations NF %%%%%%%%%%%%
    TEST_FC_1 = load('NF_TEST_T_FC_min.dat');
    TEST_FC_2 = load('NF_TEST_T_FC_max.dat');
    TEST_IC_3 = load('NF_TEST_T_IC_min.dat');
    TEST_IC_4 = load('NF_TEST_T_IC_max.dat');
    
    %Set the correct file names to store the data at the end
    file_name_train = 'SVD_TRAINING_NF.dat';
    file_name_test = 'SVD_TEST_NF.dat';
    
else
    %%%%%%%%%%% Loading the 4 outputs of the simulations %%%%%%%%%%%%
    T_FC_1 = load('TRAINING_T_FC_min.dat');
    T_FC_2 = load('TRAINING_T_FC_max.dat');
    T_IC_3 = load('TRAINING_T_IC_min.dat');
    T_IC_4 = load('TRAINING_T_IC_max.dat');
        
    %%%%%%%%%%% Loading the TEST 4 outputs of the simulations %%%%%%%%%%%%
    TEST_FC_1 = load('TEST_T_FC_min.dat');
    TEST_FC_2 = load('TEST_T_FC_max.dat');
    TEST_IC_3 = load('TEST_T_IC_min.dat');
    TEST_IC_4 = load('TEST_T_IC_max.dat');
    
    %Set the correct file names to store the data at the end
    file_name_train = 'SVD_TRAINING.dat';
    file_name_test = 'SVD_TEST.dat';
    
end

%Structures containing the 4 outputs 
m_train=struct('matrix',{T_FC_1,T_FC_2,T_IC_3,T_IC_4}); %training
m_test=struct('matrix',{TEST_FC_1,TEST_FC_2,TEST_IC_3,TEST_IC_4}); %test

num_of_outputs = size(m_train,2); %number of ouput signals to be analyzed
Ntransients_TRAINING = size(T_FC_1,1); %number of transients for each output in the TRAINING set
Ntransients_TEST = size(TEST_FC_1,1); %number of transients for each output in the TEST set

precision = 0.99999; %require precision for the SVD decomposition (for the truncation of the expansion)

final_coeff_train = []; %Preallocation of the final data strcture of the SVD coefficients for the training-set
final_coeff_test = []; %Preallocaition of the final data structure of the SVD coefficients for the test-set

%Normalized new ranges
maxNewRange = 1;
minNewRange = -1;

%Preallocation to store how many coefficients have been used for the decomposition
num_coeff = zeros(num_of_outputs,2); %First column is for TRAINING Second column if for TEST

for ii = 1:num_of_outputs
    
   %%%%%%%%%%%%%%%% Decomposition of the TRAINING set %%%%%%%%%%%%%%%%
   
   AA = m_train(ii).matrix; %Output i-esimo of the training set
   
   %Decomposition
   [coeff_train_ii,basi_ii] = fun_decomposition(AA,precision,Ntransients_TRAINING); 
   
   %Store the coefficients
   final_coeff_train = [final_coeff_train coeff_train_ii]; 
   
   %Store how many coefficients have been used for the decomposition
   num_coeff(ii,1) = size(coeff_train_ii,2);
   
   %%%%%%%%%%%%%%%% Decomposition of the TEST set %%%%%%%%%%%%%%%%
  
   BB = m_test(ii).matrix; %Output i-esimo is in

   %Decomposition
   medie = mean(BB,1); %calcolo media dei transitori
   data_reduced = BB - ones(Ntransients_TEST,1)*medie; %--> sottrarre la media ai transitori
   coeff_test_ii = data_reduced*basi_ii; %% proiezione dei transitori ridotti sulle basi ortonormali
   
   final_coeff_test = [final_coeff_test coeff_test_ii];
   
   %Store how many coefficients have been used for the decomposition
   num_coeff(ii,2) = size(coeff_test_ii,2);
   
end

display(num_coeff)

% %Save on file the SVD decomposition of the TRAINING test
% tosave = final_coeff_train;
% save(file_name_train,'tosave','-ascii');
% 
% %Save on file the SVD decomposition of the TEST test
% tosave = final_coeff_test;
% save(file_name_test,'tosave','-ascii');

    