clear
close all
clc 

set(0,'defaultlinelinewidth',2)
set(0,'defaultaxesfontsize',14)
set(0,'defaulttextfontsize',14)

%Normalization is performed

%Performs SVD decomposition in order to obtain a
%matrix which is (Ntransients x NcoeffOfSVD) which condenses all the info
%of the 4 output signals in a number of columns equal to NcoeffOfSVD

%Please choose if you want to produce the SVD decomposition with NF data or
%not, by changing the value of NF_is_on to 1 or 0
NF_is_on = 0;

if NF_is_on
    
    %%%%%%%%%%% Loading the 4 outputs of the simulations NF on %%%%%%%%%%%%
    T_FC_1 = load('NF_TRAINING_T_FC_min.dat');
    T_FC_2 = load('NF_TRAINING_T_FC_max.dat');
    T_IC_3 = load('NF_TRAINING_T_IC_min.dat');
    T_IC_4 = load('NF_TRAINING_T_IC_max.dat');
    
    %%%%%%%%%%%% Loading the TEST 4 outputs of the simulations NF %%%%%%%%%%%%
    TEST_FC_1 = load('NF_TEST_T_FC_min.dat');
    TEST_FC_2 = load('NF_TEST_T_FC_max.dat');
    TEST_IC_3 = load('NF_TEST_T_IC_min.dat');
    TEST_IC_4 = load('NF_TEST_T_IC_max.dat');
    
    %Set the correct file names to store the data at the end
    file_name_train_norm1 = 'SVD_TRAINING_NORM1_NF.dat';
    file_name_test_norm1 = 'SVD_TEST_NORM1_NF.dat';
    file_name_train_norm2 = 'SVD_TRAINING_NORM2_NF.dat';
    file_name_test_norm2 = 'SVD_TEST_NORM2_NF.dat';
    
else
    %%%%%%%%%%% Loading the 4 outputs of the simulations %%%%%%%%%%%%
    T_FC_1 = load('TRAINING_T_FC_min.dat');
    T_FC_2 = load('TRAINING_T_FC_max.dat');
    T_IC_3 = load('TRAINING_T_IC_min.dat');
    T_IC_4 = load('TRAINING_T_IC_max.dat');
        
    %%%%%%%%%%% Loading the TEST 4 outputs of the simulations %%%%%%%%%%%%
    TEST_FC_1 = load('TEST_T_FC_min.dat');
    TEST_FC_2 = load('TEST_T_FC_max.dat');
    TEST_IC_3 = load('TEST_T_IC_min.dat');
    TEST_IC_4 = load('TEST_T_IC_max.dat');
    
    %Set the correct file names to store the data at the end
    file_name_train_norm1 = 'SVD_TRAINING_NORM1.dat';
    file_name_test_norm1 = 'SVD_TEST_NORM1.dat';
    file_name_train_norm2 = 'SVD_TRAINING_NORM2.dat';
    file_name_test_norm2 = 'SVD_TEST_NORM2.dat';
    
end

%Structures containing the 4 outputs 
m_train=struct('matrix',{T_FC_1,T_FC_2,T_IC_3,T_IC_4}); %training
m_test=struct('matrix',{TEST_FC_1,TEST_FC_2,TEST_IC_3,TEST_IC_4}); %test

num_of_outputs = size(m_train,2); %number of ouput signals to be analyzed
Ntransients_TRAINING = size(T_FC_1,1); %number of transients for each output in the TRAINING set
Ntransients_TEST = size(TEST_FC_1,1); %number of transients for each output in the TEST set

precision = 0.99999; %require precision for the SVD decomposition (for the truncation of the expansion)

%Some necessary preallocation
final_coeff_train_norm1 = []; %Preallocation of the final data strcture of the SVD coefficients for the training-set
final_coeff_train_norm2 = [];
final_coeff_test_norm1 = []; %Preallocaition of the final data structure of the SVD coefficients for the test-set
final_coeff_test_norm2 = [];

%Normalized new ranges
maxNewRange = 1;
minNewRange = -1;

%Preallocation to store how many coefficients have been used for the decomposition
num_coeff_norm1 = zeros(num_of_outputs,2); %First column is for TRAINING Second column if for TEST | NORM1
num_coeff_norm2 = zeros(num_of_outputs,2); %First column is for TRAINING Second column if for TEST | NORM2

for ii = 1:num_of_outputs %do it for each output
    
   %%%%%%%%%%%%%%%% Normalization and SVD of the TRAINING set %%%%%%%%%%%%%%%%
   
   AA_to_normalize = m_train(ii).matrix; %Output i-esimo of the training set
   
   %Normalization step before SVD (norm1)
   [AA_normalized,minOR1_TRAIN,maxOR1_TRAIN] = fun_normalize_range(AA_to_normalize,minNewRange,maxNewRange); %Normalize the output signal ii
   
   %Decomposition step 
   [coeff_train_ii,basi_ii] = fun_decomposition(AA_normalized,precision,Ntransients_TRAINING); 
   
   %Store the coefficients normalized 1 time before the SVD
   final_coeff_train_norm1 = [final_coeff_train_norm1 coeff_train_ii]; 
   
   %%%% Normalization step after SVD (norm2)
   [coeff_train_ii_norm2,minOR2_TRAIN,maxOR2_TRAIN] = fun_normalize_range(coeff_train_ii,minNewRange,maxNewRange); %Normalize the output signal ii

   %Store the coefficients normalized 2 times: before and after the SVD
   final_coeff_train_norm2 = [final_coeff_train_norm2 coeff_train_ii_norm2]; 
   
   %%%%%%%%%%%%%%%% Normalization and SVD of the TEST set %%%%%%%%%%%%%%%%
   
   %Normalization step before SVD
   BB_to_normalize = m_test(ii).matrix; %Output i-esimo of the test set

   %Normalization step (notice: with the max and min (before SVD) of the TRAINING set)
   BB_normalized = minNewRange + ((maxNewRange - minNewRange)*(BB_to_normalize - minOR1_TRAIN))./(maxOR1_TRAIN - minOR1_TRAIN);
   
   %Decomposition
   medie = mean(BB_normalized,1); %calcolo media dei transitori
   data_reduced = BB_normalized - ones(Ntransients_TEST,1)*medie; %--> sottrarre la media ai transitori
   coeff_test_ii = data_reduced*basi_ii; %% proiezione dei transitori ridotti sulle basi ortonormali
   
   final_coeff_test_norm1 = [final_coeff_test_norm1 coeff_test_ii];
   
   %Normalization step (notice: with the max and min (after SVD) of the TRAINING set)
   coeff_test_ii_norm2 = minNewRange + ((maxNewRange - minNewRange)*(coeff_test_ii - minOR2_TRAIN))./(maxOR2_TRAIN - minOR2_TRAIN);
   
   %Store the coefficients normalized 2 times: before and after the SVD test
   final_coeff_test_norm2 = [final_coeff_test_norm2 coeff_test_ii_norm2]; 
   
 
   %Store how many coefficients have been used for the decomposition
   num_coeff_norm1(ii,1) = size(coeff_train_ii,2);
   num_coeff_norm1(ii,2) = size(coeff_test_ii,2);
   num_coeff_norm2(ii,1) = size(coeff_train_ii_norm2,2);
   num_coeff_norm2(ii,2) = size(coeff_test_ii_norm2,2);
   
end

disp(num_coeff_norm1)
disp(num_coeff_norm2)


% %Save on file the SVD decomposition of the TRAINING test norm1
% tosave = final_coeff_train_norm1;
% save(file_name_train_norm1,'tosave','-ascii');
% 
% %Save on file the SVD decomposition of the TRAINING test norm2
% tosave = final_coeff_train_norm2;
% save(file_name_train_norm2,'tosave','-ascii');
% 
% %Save on file the SVD decomposition of the TEST test norm1
% tosave = final_coeff_test_norm1;
% save(file_name_test_norm1,'tosave','-ascii');
% 
% %Save on file the SVD decomposition of the TEST test norm2
% tosave = final_coeff_test_norm2;
% save(file_name_test_norm2,'tosave','-ascii');
%     