clear
close all
clc

set(0,'defaultlinelinewidth',2)
set(0,'defaultaxesfontsize',14)
set(0,'defaulttextfontsize',14)

%%%%%%%%%%%%%%%%%%% What this script does %%%%%%%%%%%%%%
% 1) Classifier optimization of hyperpaameters
% 2) Test set prediction maps using the classifier
% 3) Corret or missclassified map
% 4) Confusion matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% for pp = 1:3
%
%     clearvars -except pp
%     close all
%     clc
%
%     set(0,'defaultlinelinewidth',2)
%     set(0,'defaultaxesfontsize',14)
%     set(0,'defaulttextfontsize',14)
%
%
%     cat_v = {'_C1';'_C2';'_C3'};
%
%%%%%%%%%%%%%% CONTROL PANEL %%%%%%%%%%%%%%%%%%%
data_set = '_NF'; % '' or '_NF'
categorization =  '_C3'; %cat_v{pp}; %['_C1';'_C2';'_C3'];
preprocessing = '_NORM2'; %['';'_NORM1';'_NORM2'];

%%%%%%%%%%%%%% CONTROL PANEL %%%%%%%%%%%%%%%%%%%

%Setting the location where to save and the figure names
path = 'C:\Users\NICAR\OneDrive\Desktop\Figures_classifier'; %Please change the path according to where you want to change
base_name_fig1 = '\fig_kNN_hyp_opt';
base_name_fig2 = '\fig_kNN_pred_map';
base_name_fig3 = '\fig_kNN_corr_miss';
base_name_fig4 = '\fig_kNN_conf_mat';

ext_jpg = '.jpg'; %Extension to save the picture as jpg
ext_dat = '.dat'; %Extension to load files as .dat
ext_txt = '.txt'; %Extension to load files as .txt
ext_mat = '.mat'; %Extension to save files as .mat

%Full path to save the figures
wts1 = [path base_name_fig1 categorization preprocessing data_set ext_jpg];
wts2 = [path base_name_fig2 categorization preprocessing data_set ext_jpg];
wts3 = [path base_name_fig3 categorization preprocessing data_set ext_jpg];
wts4 = [path base_name_fig4 categorization preprocessing data_set ext_jpg];

%%%%%% Input file settings %%%%%%
name_x_train = ['SVD_TRAINING' preprocessing data_set ext_dat];
name_x_test = ['SVD_TEST' preprocessing data_set ext_dat];
name_labels_train = ['labels' categorization '_TRAINING' data_set ext_txt];
name_labels_test = ['labels' categorization '_TEST' data_set ext_txt];
name_sim_test = ['input_TEST' data_set ext_txt];

category_name = [erase(categorization,'_') ')'];

%%%%%%%%%%%%%%%%%%%%%%%%%% LOADING SVD FEATURES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%% LOADING THE TRAINING-DATA-SET %%%%%%%
X_train = load(name_x_train); %Features of the training-data-set

%%%%%% LOADING THE TEST-DATA-SET %%%%%%%
X_test = load(name_x_test); %Features of the test-data-set

%%%%%%%%%%%%%%%%%%%%%%%%%% LOADING LABELS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Loading labels TRAINING SET
fileName = name_labels_train;
FID = fopen(fileName);
data = textscan(FID,'%s','Delimiter',{'\n'});
fclose(FID);
labels_TRAINING = string(data{:});
Y_train = categorical(labels_TRAINING);

%Loading labels TEST SET
fileName = name_labels_test; %Missing and needs to be created
FID = fopen(fileName);
data = textscan(FID,'%s','Delimiter',{'\n'});
labels_TEST = string(data{:});
fclose(FID);
labels_TEST = categorical(labels_TEST);

%%%%%%%%%%%%%%%%%%%%%%%%%% LOADING INPUT POINTS TEST SET %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fileName = name_sim_test;
FID = fopen(fileName);
data = textscan(FID,'%s','Delimiter',{','});
sim_codes = string(data{:});
sim_codes = reshape( sim_codes , 3 , [])';
fclose(FID);
sim_codes = str2double(sim_codes); %Convert the string matrix in a matrix of doubles

%Make the legend list accordingly
legend_list = unique(labels_TRAINING);

%% Cross validation of the data in order to optimize K and choice of the best distance

k_max = 40; %I look at alghoritms using k in the range: k=1..40

%List of distances I want to use
distances = ["euclidean" "seuclidean" "cosine" "correlation" "chebychev","minkowski" ];
s = struct('char',{'euclidean' 'seuclidean' 'cosine' 'correlation' 'chebychev' 'minkowski'});

figure(1)
hold on
for jj = 1:numel(distances)
    
    for kk=1:k_max
        Mdl = fitcknn(X_train,Y_train,'NumNeighbors',kk,'NSMethod','exhaustive','Distance',s(jj).char);
        CVMdl = crossval(Mdl);
        kloss(kk) = kfoldLoss(CVMdl);
    end
    tmp = find(kloss == min(kloss));
    k_choice(jj) = tmp(1); %The best value of k for different distances (choose the first element in case of repetitions)
    kloss_for_k_choice(jj) = min(kloss); %The value of the the classification error in the case the best k is used
    
    fig1 = plot(100*kloss,'--o')
    xlabel('K value (-)')
    ylabel('Cross-validation error (%)')
    
    
end
title(strcat('K Optimization for different distances (',category_name))
grid on
legend(distances,'Location','bestoutside')

%Save classifier optimization of hyperpaameters - fig1
saveas(fig1,wts1)

%Choice of the best distance and best k for that distance
best_dist_ind = find(kloss_for_k_choice == min(kloss_for_k_choice));
disp(s(best_dist_ind(1)).char)
k_best = k_choice(best_dist_ind(1));

%% %%%%%%%%%%% TEST OF THE CLASSIFIER %%%%%%%%%%%%%%
%Testing the trained and quality-verified classifier over a test set in
%order to see the performance on real data

%Train the classifier according to the optimized settings of k and distance
classifier_knn = fitcknn(X_train,Y_train,'NumNeighbors',k_best,'NSMethod','exhaustive','Distance',s(best_dist_ind).char);

%Perform the classification - in output there are the labels predicted for X_test
[labels_predictions,score]= predict(classifier_knn,X_test);

% Check how many predictions the classifier (kNN) got right
got_right = 100*sum(labels_predictions == labels_TEST)./length(labels_TEST);

%Plot the input points with the predicted labels
hold off

figure
view(3)
grid on
hold on

% Plot each group individually:
for kk = 1:length(legend_list)
    
    % Get indices of this particular unique group:
    ind = labels_predictions==legend_list(kk);
    
    if sum(ind)>0
        
        current_color = fun_choose_color(legend_list(kk),erase(category_name,')'));
        
        % Plot only this group:
        fig2 = plot3(-100*sim_codes(ind,1),-100*sim_codes(ind,2),-100*sim_codes(ind,3),'.','markersize',14,'displayname',legend_list(kk));
        fig2.Color = current_color;
        
    end
end
plot(0,0,'c*','MarkerSize',10,'displayname','Nominal')
legend
xlabel('\DeltaFC flow rate (%)')
ylabel('\DeltaIC flow rate (%)')
zlabel('\DeltaGC flow rate (%)')
title(strcat('kNN Prediction ( ', category_name, ' Correct = ', num2str(round(got_right,1)),'%' ))


%Save test set prediction map using the classifier - fig2
saveas(fig2,wts2)

%% Classifier Correct or Missclassified plot

% Get indices of this particular unique group:
ind_correct = labels_predictions==labels_TEST;
ind_miss = labels_predictions ~= labels_TEST;

figure
view(3)
grid on
hold on

% Plot correct and missclassified points
plot3(-100*sim_codes(ind_correct,1),-100*sim_codes(ind_correct,2),-100*sim_codes(ind_correct,3),'.g','markersize',14,'displayname','Correct');
plot3(-100*sim_codes(ind_miss,1),-100*sim_codes(ind_miss,2),-100*sim_codes(ind_miss,3),'.r','markersize',14,'displayname','Missclassified');

fig3 = plot(0,0,'c*','MarkerSize',10,'displayname','Nominal')
legend
xlabel('\DeltaFC flow rate (%)')
ylabel('\DeltaIC flow rate (%)')
zlabel('\DeltaGC flow rate (%)')
title(strcat('Test-set C/M map ( ', category_name))
%Save corret or missclassified map - fig3
saveas(fig3,wts3)


%% Confusion matrix
figure
cm = confusionchart(labels_TEST,labels_predictions,'RowSummary',...
    'row-normalized','ColumnSummary','column-normalized');
title(strcat('Confusion matrix ( ', category_name))
%Save confusion matrix - fig4
saveas(cm,wts4)


%% Table with all the details for the different classifiers
% CR_4table = CR_vec(cr);
% norm_4table = norm_vec(nn);

k_4table = k_best;
distance_4table = s(best_dist_ind).char;
gotright_4table = round(got_right,1);

%This table is for data which are not NF

TableClassifier = table;
% TableClassifier.NORM = norm_4table;
% TableClassifier.CR = CR_4table;
TableClassifier.k = k_4table;
TableClassifier.Dist = distance_4table;
TableClassifier.AM = gotright_4table;

name_table = ['TableClassifier' categorization preprocessing data_set '.mat'];
save(name_table,'TableClassifier')


