clear
close all
clc

%Generating the 4 output matrices for the TEST data set and putting labels
%of the kind NF/PF/S

set(0,'defaultlinelinewidth',2)
set(0,'defaultaxesfontsize',13)
set(0,'defaulttextfontsize',13)

%%%%%%% LOADING THE SIMULATIONS POINTS - input space %%%%%%%
fileName = 'input_TEST_NF.txt'; 
FID = fopen(fileName);
data_in = textscan(FID,'%s','Delimiter',{','});
sim_codes = string(data_in{:});
sim_codes = reshape( sim_codes , 3 , [])';
fclose(FID);

num_of_sim = size(sim_codes,1); %Number of simulations

%Time length of the transient
Ntime = 806;

%Physical constraints
T_max = 1373; %K
T_min = 858; %K

%Making the right name of the file to be loaded
name = "TEST_exp_wp_wi_wg"; %Change this if needed
extension = ".mat";

%Initialization of some counters
counter_failed = 0;
count_num_fail = 0;
counter_success = 0;
counter_physical_fail = 0;

%Initialization vector of labels - one label for each simulation.
labels = strings(num_of_sim,1);

%Looping to read all the files
for kk = 1:num_of_sim
    
    failed = 0; %Assume it has not failed
    
    cod_kk =""; %Is an empty string
    %Assemble the code of the simulation
    for jj = 1:3
        cod_kk = cod_kk+sim_codes(kk,jj);
    end
    
    %Assembling the name for the file to be red
    full_str = name + cod_kk + extension;
    sim_name = char(full_str); %I need to convert it to a character otherwise dymload will fail
    
    %Loading the file with the data
    res = dymload(sim_name);
    
    tt = dymget(res,'Time'); %time [s]
    T_FC_min = dymget(res,'Tcore_in'); %T_core_in [K]
    T_FC_max = dymget(res,'Tcore_out'); %T_core_out [K]
    
    T_IC_min = dymget(res,'T_IC_min'); %Temperature intermediate in [K]
    T_IC_max = dymget(res,'T_IC_max'); %Temperature intermediate out [K]
    
    A1 = T_FC_min;
    A2 = T_FC_max;
    A3 = T_IC_min;
    A4 = T_IC_max;
    
    
    %Logic starts here:
    
    if length(tt)<Ntime %Change this depending on the Time interval and dt of the simulation
        %Simulation has failed numerically
        count_num_fail = count_num_fail +1;
        labels(kk) = "Numerical Failure";
        
        %If failed numerically, artifically prolong the array with the last
        %value of the output
        n_miss = Ntime - length(tt);
        v_one = ones(n_miss,1);
        A1 = [T_FC_min; T_FC_min(end)*v_one];
        A2 = [T_FC_max; T_FC_max(end)*v_one];
        A3 = [T_IC_min; T_IC_min(end)*v_one];
        A4 = [T_IC_max; T_IC_max(end)*v_one];
        
    else
        %Simulation could be failed physically or be succesfull
        
        if sum(find(T_FC_max>T_max))>0 || sum(find(T_FC_min>T_max))>0 %Check Tmax in FC
            %Simulation has failed physically
            
            labels(kk) = "Physical Failure";
        end
        
        if sum(find(T_FC_min<T_min))>0 || sum(find(T_FC_max<T_min))>0
            %Simulation has failed physically
            
            labels(kk) = "Physical Failure";
        end
        
        if sum(find(T_IC_max>T_max))>0 || sum(find(T_IC_min>T_max))>0
            %Simulation has failed physically
            
            labels(kk) = "Physical Failure";
        end
        
        if sum(find(T_IC_min<T_min))>0 || sum(find(T_IC_max<T_min))>0
            %Simulation has failed physically
            labels(kk) = "Physical Failure";
        end  
        
    end
    
    if labels(kk) == "" %If at this point nothing was written -> sim is succesfull
        labels(kk) = "Success";
        counter_success = counter_success + 1;
    else %sim has failed somehow (phys or num)
        counter_failed = counter_failed +1;
    end
    
    %Storing the output of all the simulations
    T1(kk,:) = A1;
    T2(kk,:) = A2;
    T3(kk,:) = A3;
    T4(kk,:) = A4;
    
end

counter_physical_fail = counter_failed - count_num_fail;

disp(counter_failed)            % 499 failures
disp(count_num_fail)            % 220 are numerical
disp(counter_physical_fail)     % 279 are ONLY physical failure
disp(counter_success)           % 501 simulations are of success

%To counter check
% a = sum(labels == "Numerical Failure")
% b = sum(labels == "Physical Failure")
% c = sum(labels == "Success")

%Now I remove from the data the failed simulations
ii = 1;
for kk = 1:num_of_sim
    
    if labels(kk) == "Numerical Failure"
        %Don't store
    else
        %Store
        data(ii,:) = sim_codes(kk,:);
        new_labels(ii,1) = labels(kk);
        
        TT_FC_min(ii,:) = T1(kk,:);
        TT_FC_max(ii,:) = T2(kk,:);
        TT_IC_min(ii,:) = T3(kk,:);
        TT_IC_max(ii,:) = T4(kk,:);
        
        
        ii = ii +1; %update index to store next data
        
    end
end


%% Write on file the output

%Writing the input
writematrix(data,'input_TEST.txt');

%Writing the labels
fid = fopen('labels_C1_TEST.txt','w');
for jj = 1:size(new_labels,1)
    fprintf( fid, '%s\n', new_labels(jj) );
end
fclose(fid);

%Writing the 4 non-failed outputs
num_of_transients = 4; %As I'm considering 4 output parameters at the moment, change this number if needed
m=struct('matrix',{TT_FC_min,TT_FC_max,TT_IC_min,TT_IC_max});
s = struct('char',{'TEST_T_FC_min.dat' 'TEST_T_FC_max.dat' 'TEST_T_IC_min.dat' 'TEST_T_IC_max.dat'});

for ii = 1:num_of_transients
tosave = m(ii).matrix;
save(s(ii).char,'tosave','-ascii');
end

%% Storing also the Failed data artifically prolonged

%Writing the input - it's the full input_TEST_NF file

%Writing the labels
fid = fopen('labels_C1_TEST_NF.txt','w');
for jj = 1:size(labels,1)
    fprintf( fid, '%s\n', labels(jj) );
end
fclose(fid);

%Writing the 4 outputs
num_of_transients = 4; %As I'm considering 4 output parameters at the moment, change this number if needed
m_NF=struct('matrix',{T1,T2,T3,T4});
s_NF = struct('char',{'NF_TEST_T_FC_min.dat' 'NF_TEST_T_FC_max.dat' 'NF_TEST_T_IC_min.dat' 'NF_TEST_T_IC_max.dat'});

for ii = 1:num_of_transients
tosave = m_NF(ii).matrix;
save(s_NF(ii).char,'tosave','-ascii');
end