clear
close all
clc

set(0,'defaultlinelinewidth',2)
set(0,'defaultaxesfontsize',14)
set(0,'defaulttextfontsize',14)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% CONTROL PANEL %%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Choose if NF is on:
ifNF = ''; % '_NF' (on) or '' (off)

%Choose if we're loading training or test
TRAINING = 0; %1 for TRAINING - 0 for TEST

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% CONTROL PANEL %%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Name of files where to save
ext = '.txt';
name_test = ['labels_C2_TEST' ifNF ext];
name_training = ['labels_C2_TRAINING' ifNF ext];

if ifNF == "_NF"
    
    if TRAINING
        
        %%%%%%%%%%% Loading the 4 outputs of the simulations NF %%%%%%%%%%%%
        is_TEST = 0;
        T_FC_1 = load('NF_TRAINING_T_FC_min.dat');
        T_FC_2 = load('NF_TRAINING_T_FC_max.dat');
        T_IC_3 = load('NF_TRAINING_T_IC_min.dat');
        T_IC_4 = load('NF_TRAINING_T_IC_max.dat');
    else
        
        %%%%%%%%%%%% Loading the TEST 4 outputs of the simulations NF %%%%%%%%%%%%
        is_TEST = 1;
        T_FC_1 = load('NF_TEST_T_FC_min.dat');
        T_FC_2 = load('NF_TEST_T_FC_max.dat');
        T_IC_3 = load('NF_TEST_T_IC_min.dat');
        T_IC_4 = load('NF_TEST_T_IC_max.dat');
    end
else
    if TRAINING
        %%%%%%%%%% Loading the 4 outputs of the simulations %%%%%%%%%%%%
        is_TEST = 0;
        T_FC_1 = load('TRAINING_T_FC_min.dat');
        T_FC_2 = load('TRAINING_T_FC_max.dat');
        T_IC_3 = load('TRAINING_T_IC_min.dat');
        T_IC_4 = load('TRAINING_T_IC_max.dat');
    else
        %%%%%%%%%%% Loading the TEST 4 outputs of the simulations %%%%%%%%%%%%
        is_TEST = 1;
        T_FC_1 = load('TEST_T_FC_min.dat');
        T_FC_2 = load('TEST_T_FC_max.dat');
        T_IC_3 = load('TEST_T_IC_min.dat');
        T_IC_4 = load('TEST_T_IC_max.dat');
    end
end

%Physical constraints
T_max = 1373; %K
T_min = 858; %K

dim_in = 3;

%Number of simulations
num_of_sim = size(T_FC_1,1);

%List of possible FFL accidents
legend_list = ["HT_{FC}","LT_{FC}","HT_{IC}","LT_{IC}"];

%Looping to read all the files
for kk = 1:num_of_sim
    
    T_FC_min = T_FC_1(kk,:)';
    T_FC_max = T_FC_2(kk,:)';
    T_IC_min = T_IC_3(kk,:)';
    T_IC_max = T_IC_4(kk,:)';

    ind_FC_min_1 = find(T_FC_min<T_min);
    ind_FC_min_2 = find(T_FC_max<T_min); 
    
    ind_FC_max_1 = find(T_FC_min>T_max);
    ind_FC_max_2 = find(T_FC_max>T_max);
    
    ind_IC_min_1 = find(T_IC_min<T_min);
    ind_IC_min_2 = find(T_IC_max<T_min);
    
    ind_IC_max_1 = find(T_IC_min>T_max);
    ind_IC_max_2 = find(T_IC_max>T_max);
    
    v1 = [ind_FC_max_1;ind_FC_max_2]; %1 Tmax FC
    v2 = [ind_FC_min_1;ind_FC_min_2]; %2 Tmin FC
    v3 = [ind_IC_max_1;ind_IC_max_2]; %3 Tmax IC
    v4 = [ind_IC_min_1;ind_IC_min_2]; %4 Tmin IC

    if ~isempty(v1) || ~isempty(v2) || ~isempty(v3) || ~isempty(v4) %If the simulation has failed 
        
        if isempty(v1)
            v1 = NaN;
        end
        if isempty(v2)
            v2 = NaN;
        end
        if isempty(v3)
            v3 = NaN;
        end
        if isempty(v4)
            v4 = NaN;
        end
        
        vv = [min(v1),min(v2),min(v3),min(v4)];
        first_failure = min(vv);
        
        %First to fail is
        pos_FF = find(vv==first_failure); %this is a number between 1 and 4
        
        labels(kk) = legend_list(pos_FF);
        %1 Tmax FC
        %2 Tmin FC
        %3 Tmax IC
        %4 Tmin IC
    else
        %Success
        labels(kk) = "Success";
    end
   

end

%Check the numbers are coherent
disp("Success")
disp(sum(labels=="Success"));

for ii = 1:numel(legend_list)
    disp(legend_list(ii))
    disp(sum(labels==legend_list(ii)));
end

if is_TEST
    %Writing the labels
    fid = fopen(name_test,'w');
    for jj = 1:numel(labels)
        fprintf( fid, '%s\n', labels(jj) );
    end
    fclose(fid);
else

    %Writing the labels
    fid = fopen(name_training,'w');
    for jj = 1:numel(labels)
        fprintf( fid, '%s\n', labels(jj) );
    end
    fclose(fid);

end