clear
close all
clc

set(0,'defaultlinelinewidth',2)
set(0,'defaultaxesfontsize',14)
set(0,'defaulttextfontsize',14)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% CONTROL PANEL %%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Choose if NF is on:
ifNF = ''; % '_NF' (on) or '' (off)

%Choose if we're loading training or test
TRAINING = 0; %1 for TRAINING - 0 for TEST

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% CONTROL PANEL %%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Name of files where to save
ext = '.txt';
name_test = ['labels_C3_TEST' ifNF ext];
name_training = ['labels_C3_TRAINING' ifNF ext];

if ifNF == "_NF"
    
    if TRAINING
        
        %%%%%%%%%%% Loading the 4 outputs of the simulations NF %%%%%%%%%%%%
        is_TEST = 0;
        T_FC_1 = load('NF_TRAINING_T_FC_min.dat');
        T_FC_2 = load('NF_TRAINING_T_FC_max.dat');
        T_IC_3 = load('NF_TRAINING_T_IC_min.dat');
        T_IC_4 = load('NF_TRAINING_T_IC_max.dat');
    else
        
        %%%%%%%%%%%% Loading the TEST 4 outputs of the simulations NF %%%%%%%%%%%%
        is_TEST = 1;
        T_FC_1 = load('NF_TEST_T_FC_min.dat');
        T_FC_2 = load('NF_TEST_T_FC_max.dat');
        T_IC_3 = load('NF_TEST_T_IC_min.dat');
        T_IC_4 = load('NF_TEST_T_IC_max.dat');
    end
else
    if TRAINING
        %%%%%%%%%% Loading the 4 outputs of the simulations %%%%%%%%%%%%
        is_TEST = 0;
        T_FC_1 = load('TRAINING_T_FC_min.dat');
        T_FC_2 = load('TRAINING_T_FC_max.dat');
        T_IC_3 = load('TRAINING_T_IC_min.dat');
        T_IC_4 = load('TRAINING_T_IC_max.dat');
    else
        %%%%%%%%%%% Loading the TEST 4 outputs of the simulations %%%%%%%%%%%%
        is_TEST = 1;
        T_FC_1 = load('TEST_T_FC_min.dat');
        T_FC_2 = load('TEST_T_FC_max.dat');
        T_IC_3 = load('TEST_T_IC_min.dat');
        T_IC_4 = load('TEST_T_IC_max.dat');
    end
end

%Physical constraints
T_max = 1373; %K
T_min = 858; %K

dim_in = 3;

%Number of simulations
num_of_sim = size(T_FC_1,1);

%List of basic accidents
basic_accidents = ["HT_{FC}","LT_{FC}","HT_{IC}","LT_{IC}"];
legend_list =fun_make_comb(basic_accidents);

m = struct('matrix',{T_FC_1,T_FC_2,T_IC_3,T_IC_4});
T = struct('scalar',{T_max,T_min});

logic = m(1).matrix>T(1).scalar | m(2).matrix>T(1).scalar;
cc = sum(logic,2);
CC(:,1) = cc>1;

logic = m(1).matrix<T(2).scalar | m(2).matrix<T(2).scalar;
cc = sum(logic,2);
CC(:,2) = cc>1;

logic = m(3).matrix>T(1).scalar | m(4).matrix>T(1).scalar;
cc = sum(logic,2);
CC(:,3) = cc>1;

logic = m(3).matrix<T(2).scalar | m(4).matrix<T(2).scalar;
cc = sum(logic,2);
CC(:,4) = cc>1;


for ii = 1:num_of_sim
    
    aa = basic_accidents(CC(ii,:));
    current_accident = "";
    for jj = 1:numel(aa)
        current_accident = current_accident + aa(jj);
    end
    
    labels(ii,1) = current_accident;

end
labels(labels == "") = "Success";

%%%%%%%%%%%%%%%%%%%%% Check the numbers are coherent %%%%%%%%%%%%%%%%%%%%% 
disp(labels)

disp("Success")
disp(sum(labels=="Success"));

for ii = 1:numel(legend_list)
    disp(legend_list(ii))
    disp(sum(labels==legend_list(ii)));
end


if is_TEST
    %%%%%%%%%%%%%%% Writing the labels on file %%%%%%%%%%%%%%%
    fid = fopen(name_test,'w');
    for jj = 1:numel(labels)
        fprintf( fid, '%s\n', labels(jj) );
    end
    fclose(fid);
    
else
    %%%%%%%%%%%%%%% Writing the labels on file %%%%%%%%%%%%%%%
    fid = fopen(name_training,'w');
    for jj = 1:numel(labels)
        fprintf( fid, '%s\n', labels(jj) );
    end
    fclose(fid);
    
end

